@@ -193,7 +193,7 @@ void open_fifo()
     struct stat sbuf;
 
     if (fifoname) {
-	if (stat(fifoname,&sbuf) < 0 || !sbuf.st_mode&S_IFIFO) {
+	if (stat(fifoname,&sbuf) < 0 || !(sbuf.st_mode&S_IFIFO)) {
 	    syslog(LOG_INFO,"Creating FIFO");
 	    /* Create the fifo. */
 	    mknod(fifoname, S_IFIFO|0277, 0);
@@ -208,10 +208,11 @@ void open_fifo()
 	if ((fifo_fd = open(fifoname, O_RDWR | O_NDELAY)) >= 0) {
 	    if (debug&DEBUG_VERBOSE)
 	   	 syslog(LOG_INFO,"Using fifo %s",fifoname);
+	    pipe_init(fifo_fd,&fifo_pipe);
 	} else {
 	    syslog(LOG_ERR,"Could not open fifo file %s",fifoname);
+	    fifo_fd = -1;
 	}
-	pipe_init(fifo_fd,&fifo_pipe);
     } else {
 	/* make sure to invalidate the fifo_fd if we don't open one. */
 	fifo_fd = -1;
@@ -341,7 +342,11 @@ void fifo_read()
 
     i = pipe_read(&fifo_pipe);
     buf = tail = fifo_pipe.buf;
-    if (i <= 0) return;	/* FIXME: swallow errors */
+    if (i < 0) {
+	fifo_fd = -1;
+	return;
+    }
+    if (i == 0) return;
 
     while (i--) {
         if (*tail == '\n') {
@@ -401,21 +406,43 @@ void fifo_read()
 		int fd;
 		MONITORS *new;
 
-		if (buf[7] != 0 && buf[7] == ' ') {
-		    syslog(LOG_INFO,"FIFO: monitor connection to %s requested",
-			    buf+8);
-		    if (stat(fifoname,&sbuf) < 0 || !sbuf.st_mode&S_IFIFO) {
-			syslog(LOG_INFO,"FIFO: %s not a pipe.",
-			    buf+8);
-		    } else if ((fd = open(buf+8,O_WRONLY))<0) {
-			syslog(LOG_INFO,"FIFO: could not open pipe %s: %m",
-			    buf+8);
-		    } else {
-			new = (MONITORS *)malloc(sizeof(MONITORS));
-			new->next = monitors;
-			new->fd = fd;
-			monitors = new;
-			output_state();
+		k = 0;
+		if (sscanf(buf,"monitor %d %n",&j,&k) == 1) {
+		    syslog(LOG_INFO,"FIFO: monitor connection at info level %d to %s requested",
+			    j,buf+k);
+		} else if (buf[7] != 0 && buf[7] == ' ') {
+		    syslog(LOG_INFO,"FIFO: full monitor connection to %s requested",
+			buf+k);
+		    j = 255;	/* Heavy weight connection requested */
+		    k = 8;
+		}
+		if (k >= 8) {
+		    /* Check list to see if this is just a status change */
+		    new = monitors;
+		    while (new) {
+			if (strcmp(new->name,buf+k) == 0) {
+			    new->level = j;
+			    output_state();
+			    break;
+			}
+			new = new->next;
+		    }
+		    if (!new) {
+			if (stat(fifoname,&sbuf) < 0 || !sbuf.st_mode&S_IFIFO) {
+			    syslog(LOG_INFO,"FIFO: %s not a pipe.",
+				buf+k);
+			} else if ((fd = open(buf+k,O_WRONLY))<0) {
+			    syslog(LOG_INFO,"FIFO: could not open pipe %s: %m",
+				buf+k);
+			} else {
+			    new = (MONITORS *)malloc(sizeof(MONITORS));
+			    new->name = strdup(buf+k);
+			    new->next = monitors;
+			    new->fd = fd;
+			    new->level = j;
+			    monitors = new;
+			    output_state();
+			}
 		    }
 		} else {
 		    syslog(LOG_INFO,"FIFO: empty monitor request ignored");
@@ -423,9 +450,9 @@ void fifo_read()
 	    } else if (strncmp(buf,"message ",8) == 0) {
 		/* pass a message from the connector on to the monitor */
 		if (monitors) {
-		    mon_write("MESSAGE\n",8);
-		    mon_write(buf+8,strlen(buf+8));
-		    mon_write("\n",1);
+		    mon_write(MONITOR_MESSAGE,"MESSAGE\n",8);
+		    mon_write(MONITOR_MESSAGE,buf+8,strlen(buf+8));
+		    mon_write(MONITOR_MESSAGE,"\n",1);
 		}
             } else if (sscanf(buf,"connect %d %n", &pid, &dev) == 1) {
                 if (pid > 1) {
@@ -467,9 +494,11 @@ void proxy_read()
 {
     char buffer[4096];
     int len;
+    struct SOCKADDR to;
 
     /* read the SLIP packet */
     len = recv_packet(buffer,4096);
+
     if (!do_reroute) {
 	/* if we are doing unsafe routing, all counting is in the filter.
 	 * otherwise we can see transmitted bytes directly at this spot.
@@ -491,9 +520,29 @@ void proxy_read()
      * the kernel is retransmitting something through sl0, despite
      * the existance of a route through another device...
      */
-    if (link_iface != -1 && fwdfd != -1 && !(do_reroute && mode == MODE_PPP)) {
-	/* if the external iface is up then we can send it on */
-	struct SOCKADDR to;
+
+    /* if the external iface is up then probably we can send it on */
+    if (link_iface != -1 && fwdfd != -1) {
+	/* Make sure we try to restore the link to working condition now... */
+	if (do_reroute && mode == MODE_PPP) {
+	    /* Check if a route exists at this point through the ppp device. */
+	    /* If not then we must be half dead. */
+	    if (!ppp_route_exists()) {
+		/* The external iface is down, buffer the packet so we can
+	 	 * forward it when the iface comes up.
+	  	 */
+	        ppp_half_dead = 1;
+		if (buffer_packets)
+	    	    buffer_packet(len,buffer);
+		return;
+	    }
+	}
+
+	/* Ok, the interface is there, and the route is up,
+  	 * so just send it on. This can happen when routing is switched
+	 * in the middle of a retransmission sequence. (There is a race
+	 * between the route switching and the forwarding I think.)
+	 */
 
 #ifdef HAS_SOCKADDR_PKT
 	to.spkt_family = AF_INET;
@@ -510,19 +559,9 @@ void proxy_read()
 		"Error forwarding data packet to physical device: %m");
 	}
     } else {
-	/* Make sure we try to restore the link to working condition now... */
-	if (link_iface != -1 && fwdfd != -1 && do_reroute && mode == MODE_PPP) {
-	    /* Check if a route exists at this point through the ppp device. */
-	    /* If not then we must be half dead. */
-	    if (!ppp_route_exists())
-	        ppp_half_dead = 1;
-	}
-#ifdef BUFFER_PACKETS
-	/* The external iface is down, buffer the packet so we can
-	 * forward it when the iface comes up.
-	 */
-	buffer_packet(len,buffer);
-#endif
+	/* If the link isn't up, then we better buffer the packets */
+	if (buffer_packets)
+	    buffer_packet(len,buffer);
     }
 }
 
@@ -552,6 +591,7 @@ void die(int i)
 	/* Give the system a second to send the signals */
 	if (link_pid || dial_pid || running_pid) sleep(1);
 	close_modem();
+	interface_down();
     	proxy_down();
 	unlink(pidfile);
     	exit(i);
@@ -609,8 +649,13 @@ void sig_chld(int sig)
 			: "other",
 		    status);
 	if (pid == link_pid) link_pid = 0;
-	if (pid == dial_pid) { dial_status = status; dial_pid = 0; }
-	if (pid == running_pid) { running_status = status; running_pid = 0; }
+	else if (pid == dial_pid) { dial_status = status; dial_pid = 0; }
+	else if (pid == running_pid) { running_status = status; running_pid = 0; }
+	else if (!WIFEXITED(status))
+   	    syslog(LOG_ERR,"Abnormal exit (status %d) on pid %d",status,pid);
+	else if (WEXITSTATUS(status) != 0)
+	    syslog(LOG_ERR,"Nonzero exit status (%d) on pid %d",
+		WEXITSTATUS(status),pid);
 	if (pid > 0) {
 	    if (WIFSIGNALED(status)) {
 		syslog(LOG_WARNING, "child process %d terminated with signal %d",
@@ -650,11 +695,11 @@ int system(const char *buf)
     int fd;
 
     block_signals();
-    running_pid = fork();
-
     if (debug&DEBUG_VERBOSE)
 	syslog(LOG_DEBUG,"running '%s'",buf);
 
+    running_pid = fork();
+
     if (running_pid != 0) unblock_signals();
 
     if (running_pid < 0) {
@@ -676,7 +721,7 @@ int system(const char *buf)
 	close(0);
 	close(1);
 	close(2);
-	close(modem_fd);
+	if (modem_fd >= 0) close(modem_fd);
 	close(proxy_mfd);      /* close the master pty endpoint */
 	close(proxy_sfd);      /* close the slave pty endpoint */
 	if (fifo_fd != -1) close(fifo_fd);
@@ -710,23 +755,92 @@ int system(const char *buf)
     return running_status;
 }
 
-void mon_write(char *message,int len)
+void background_system(const char *buf)
+{
+    int fd, pid;
+
+    block_signals();
+    if (debug&DEBUG_VERBOSE)
+	syslog(LOG_DEBUG,"running '%s'",buf);
+
+    pid = fork();
+
+    if (pid != 0) unblock_signals();
+
+    if (pid < 0) {
+        syslog(LOG_ERR, "failed to fork and run '%s': %m",buf);
+	return;
+    }
+
+    if (pid == 0) {
+        /* change the signal actions back to the defaults, then unblock them. */
+        default_sigacts();
+	unblock_signals();
+
+        /* Leave the current location */
+        (void) setsid();    /* No controlling tty. */
+        (void) umask (S_IRWXG|S_IRWXO);
+        (void) chdir ("/"); /* no current directory. */
+
+	/* close all fd's the child should not see */
+	close(0);
+	close(1);
+	close(2);
+	if (modem_fd >= 0) close(modem_fd);
+	close(proxy_mfd);      /* close the master pty endpoint */
+	close(proxy_sfd);      /* close the slave pty endpoint */
+	if (fifo_fd != -1) close(fifo_fd);
+	if (monitors) {
+	    MONITORS *c = monitors;
+	    while (c) {
+		close(c->fd);
+		c = c->next;
+	    }
+	}
+
+	/* make sure the stdin, stdout and stderr get directed to /dev/null */
+	fd = open("/dev/null", O_RDWR);
+        if (fd >= 0) {
+	    if (fd != 0) {
+	    	dup2(fd, 0);
+		close(fd);
+	    }
+	    dup2(0, 1);
+            dup2(0, 2);
+        }
+
+        execl("/bin/sh", "sh", "-c", buf, (char *)0);
+        syslog(LOG_ERR, "could not exec /bin/sh: %m");
+        _exit(127);
+        /* NOTREACHED */
+    }
+}
+
+void mon_write(int level, char *message,int len)
 {
     MONITORS *c = monitors, *p = 0, *cn;
     while (c) {
 	cn = c->next;
-	if (write(c->fd,message,len) < 0) {
-	    if (errno == EPIPE) {
-	        syslog(LOG_INFO,"Monitor pipe closed.");
+	if (c->level&level) {
+	    if (write(c->fd,message,len) < 0) {
+		if (errno == EPIPE) {
+		    syslog(LOG_INFO,"Monitor pipe %s closed.",c->name);
+		} else {
+		    /* Write error. The reader probably got swapped out
+		     * or something and the pipe flooded. We'll just "loose"
+		     * the data.
+		     */
+		     p = c;
+		     continue;
+		}
+		close(c->fd);
+		if (p) p->next = c->next;
+		else monitors = c->next;
+		free(c->name);
+		free(c);
 	    } else {
-	        syslog(LOG_ERR,"Write error on monitor pipe. Closing pipe.");
+		p = c;
 	    }
-	    close(c->fd);
-	    if (p) p->next = c->next;
-	    else monitors = c->next;
-	    free(c);
-	} else {
-	    p = c;
 	}
 	c = cn;
     }
