@@ -46,8 +46,8 @@
 #include <arpa/inet.h>
 /* Hmm. Should there be a netinet pointer to these??? */
 #include <linux/if_ether.h>
+#include <linux/if_slip.h>
 
-#if 0  /* Darn.  This seems to have been removed in later versions */
 #include <linux/version.h>
 /* This only exists in kernels >= 1.3.75 */
 #if LINUX_VERSION_CODE >= 66379
@@ -58,12 +58,6 @@
 #include <linux/socket.h>
 #define SOCKADDR sockaddr
 #endif
-#endif
-/* assume this for later kenels */
-#define HAS_SOCKADDR_PKT
-#include <linux/if_packet.h>
-#define SOCKADDR sockaddr_pkt
-
 
 #include "config.h"
 #include "fsm.h"
@@ -82,6 +76,7 @@
 /* Operation modes */
 #define MODE_SLIP 0
 #define MODE_PPP 1
+#define MODE_DEV 2
 
 
 /* Dynamic slip interpretation modes */
@@ -93,13 +88,75 @@
 
 /* Define DEBUG flags */
 #define DEBUG_FILTER_MATCH	0x0001
-#define DEBUG_FILTER_CONFIG	0x0002
-#define DEBUG_TIMEOUT_QUEUE	0x0004
+#define DEBUG_PROXYARP		0x0004
 #define DEBUG_VERBOSE		0x0008
 #define DEBUG_STATE_CONTROL	0x0010
 #define DEBUG_TICK		0x0020
 #define DEBUG_CONNECTION_QUEUE	0x0040
-#define DEBUG_PROXYARP		0x0080
+
+/* Define MONITOR flags */
+#define MONITOR_STATE		0x0001
+#define MONITOR_INTERFACE	0x0002
+#define MONITOR_STATUS		0x0004
+#define MONITOR_LOAD		0x0008
+#define MONITOR_MESSAGE		0x0010
+#define MONITOR_QUEUE		0x0020
+
+/*
+ * If you choose UNSAFE_ROUTING=0, then by default diald will route all
+ * outgoing packets to the proxy device and forward them to the real
+ * device by itself. This has the advantage that it gets around a bug
+ * in the current production release (1.2.X) linux kernels
+ * that causes TCP sessions to lock up if the route is changed while
+ * a packet is being retransmitted. However, this introduces quite
+ * a bit of overhead on outgoing packets (10-20%). (Note that incoming packets
+ * don't go through this process!)
+ * If you choose UNSAFE_ROUTING=1, then diald will change the routes anyway.
+ * If you are using diald on a machine were there is a single outgoing
+ * link this is perfectly safe (I think!), but if diald is being used
+ * in an environment where more than one ppp or slip link can be active
+ * at a time, then there is a small chance that you can lock up TCP
+ * sessions. In particular if the link is terminated (either by diald
+ * or by the other end hanging up) when a TCP session is in the middle
+ * of retransmitting a packet, then that TCP session can become locked
+ * if when the link comes back up it is brought back up on a different
+ * ppp or slip device.
+ * [NOTE: As of linux 1.3.13, the Linux kernel has been
+ *  fixed to allow routing changes under an active TCP retransmit,
+ *  so with 1.3.13 and later UNSAFE_ROUTING as the default is perfectly safe.]
+ * [NOTE 2: unlike previous versions of diald, this option can now be
+ * controlled from the command line or configuration file. See the new
+ * options "reroute" and "-reroute".
+ */
+
+#define UNSAFE_ROUTING 1	/* do rerouting by default */
+
+/*
+ * Originally diald just threw away any packets it received when
+ * the link was down. This is OK because IP is an unreliable protocol,
+ * so applications will resend packets when the link comes back up.
+ * On the other hand the kernel doubles the timeout for TCP packets
+ * every time a send fails. If you define BUFFER_PACKETS diald
+ * will store packets that come along when the link is down and
+ * send them as soon as the link comes up. This should speed up
+ * the initial connections a bit.
+ */
+
+#define BUFFER_PACKETS 1	/* turn on packet buffering code. */
+#define BUFFER_SIZE 65536	/* size of buffer to store packets */
+#define BUFFER_FIFO_DISPOSE 1	/* dispose of old packets to make room
+				 * for new packets if the buffer becomes
+				 * full. Without this option new packets
+				 * are discarded if there is no room.
+				 */
+#define BUFFER_TIMEOUT 600	/* Maximum number of seconds to keep a
+				 * packet in the buffer. Don't make this
+				 * too large or you will break IP.
+				 * (Something on the order of 1 hour
+				 * probably the maximum safe value.
+				 * I expect that the 10 minutes defined
+				 * by default should be plenty.
+				 */
 
 /*
  * Various timeouts and times used in diald.
@@ -117,14 +174,21 @@
 typedef struct monitors {
     struct monitors *next;
     int fd;			/* monitor output fp. */
+    int level;			/* Information level requested */
+    char *name;
 } MONITORS;
 
 /* Configuration variables */
 
 char **devices;
 int device_count;
+char device[10];
+char device_node[9];
+int device_iface;
 int inspeed;
+int window;
 int mtu;
+int mru;
 char *connector;
 char *disconnector;
 char *orig_local_ip;
@@ -140,6 +204,19 @@ char *ip_down;
 char *acctlog;
 char *pidlog;
 char *fifoname;
+char *lock_prefix;
+int pidstring;
+char *run_prefix;
+char *diald_config_file;
+char *diald_defs_file;
+char *path_route;
+char *path_ifconfig;
+char *path_bootpc;
+char *path_pppd;
+int buffer_packets;
+int buffer_size;
+int buffer_fifo_dispose;
+int buffer_timeout;
 FILE *acctfp;
 int call_start_time;
 int mode;
@@ -174,6 +251,15 @@ int two_way;
 int give_way;
 int do_reroute;
 int proxyarp;
+int route_wait;
+
+#ifdef SIOCSKEEPALIVE
+extern int keepalive;
+#endif
+
+#ifdef SIOCSOUTFILL
+extern int outfill;
+#endif
 
 /* Global variables */
 
@@ -282,6 +368,7 @@ void parse_bringup(void *var, char **argv);
 void parse_keepup(void *var, char **argv);
 void parse_accept(void *var, char **argv);
 void parse_ignore(void *var, char **argv);
+void parse_wait(void *var, char **argv);
 void parse_up(void *var, char **argv);
 void parse_down(void *var, char **argv);
 void parse_prule(void *var, char **argv);
@@ -298,6 +385,7 @@ void ppp_stop(void);
 void ppp_reroute(void);
 void ppp_kill(void);
 void ppp_zombie(void);
+int ppp_rx_count(void);
 void slip_start(void);
 int slip_set_addrs(void);
 int slip_dead(void);
@@ -305,10 +393,22 @@ void slip_stop(void);
 void slip_reroute(void);
 void slip_kill(void);
 void slip_zombie(void);
+int slip_rx_count(void);
+void dev_start(void);
+int dev_set_addrs(void);
+int dev_dead(void);
+void dev_stop(void);
+void dev_reroute(void);
+void dev_kill(void);
+void dev_zombie(void);
+int dev_rx_count(void);
 void idle_filter_init(void);
 void interface_up(void);
 void interface_down(void);
+void buffer_init(int *, char **);
 int queue_empty(void);
+int fw_wait(void);
+int fw_reset_wait(void);
 int next_alarm(void);
 void buffer_packet(unsigned int,unsigned char *);
 void forward_buffer(void);
@@ -323,4 +423,7 @@ void pipe_flush(PIPE *, int);
 int set_proxyarp (unsigned int);
 int clear_proxyarp (unsigned int);
 int report_system_result(int,char *);
-void mon_write(char *,int);
+void mon_write(int,char *,int);
+void background_system(const char *);
+void block_timer();
+void unblock_timer();
