@@ -158,8 +158,11 @@ static FW_Connection *find_connection(FW_unit *unit, FW_ID *id)
 	   break;
 	c = c->next;
     }
-    if (c == unit->connections) return 0;
-    else return c;
+    if (c == unit->connections) {
+	return 0;
+    } else {
+	return c;
+    }
 }
 
 /*
@@ -463,10 +466,13 @@ int check_firewall(int unitnum, unsigned char *pkt, int len)
     FW_Connection *conn;
     struct iphdr * ip_pkt = (struct iphdr *)pkt;
 
+    block_timer();
+
     if (!initialized) init_units();
 
     if (unitnum < 0 || unitnum >= FW_NRUNIT) {
 	/* FIXME: set an errorno? */
+	unblock_timer();
 	return -1;
     }
 
@@ -483,7 +489,10 @@ int check_firewall(int unitnum, unsigned char *pkt, int len)
 	break;
     }
 
-    if (pprule == 0) return -1;	/* No protocol rules? */
+    if (pprule == 0) {
+	unblock_timer();
+	return -1;	/* No protocol rules? */
+    }
 
     /* Build the connection ID, and set the direction flag */
     direction = ip_direction(ip_pkt);
@@ -593,11 +602,21 @@ int check_firewall(int unitnum, unsigned char *pkt, int len)
 	    log_packet(fw->filt.type!=FW_TYPE_IGNORE,ip_pkt,len,rule);
 
 	/* Check if this entry goes into the queue or not */
-	if (fw->filt.type != FW_TYPE_IGNORE) {
+	if (fw->filt.type != FW_TYPE_IGNORE && fw->filt.type != FW_TYPE_WAIT) {
 	    add_connection(unit,conn,&id,fw->filt.timeout,lflags);
 	}
+	/* check if we are no longer waiting */
+	if (fw->filt.type == FW_TYPE_WAIT) {
+		unit->waiting = 0;
+		/* WAITING rules don't do the final match, but
+		 * must occur before other rules
+		 */
+		goto next_rule;
+	}
+
+	unblock_timer();
 	/* Return 1 if accepting rule with non zero timeout, 0 otherwise */
-	return (fw->filt.type != FW_TYPE_IGNORE && fw->filt.timeout > 0);
+	return ((fw->filt.type != FW_TYPE_IGNORE || fw->filt.type != FW_TYPE_WAIT) && fw->filt.timeout > 0);
 
 next_rule: /* try the next filter */
 	fw = fw->next;
@@ -606,14 +625,15 @@ next_rule: /* try the next filter */
     /* Failed to match any rule. This means we ignore the packet */
     if (debug&DEBUG_FILTER_MATCH)
         log_packet(0,ip_pkt,len,0);
+    unblock_timer();
     return 1;
 }
 
-static void pcountdown(long secs)
+static void pcountdown(int level, long secs)
 {
     char buf[10];
     sprintf(buf,"%02ld:%02ld:%02ld\n",secs/3600,(secs/60)%60,secs%60);
-    if (monitors) mon_write(buf,9);
+    if (monitors) mon_write(level,buf,9);
 }
 
 int ctl_firewall(int op, struct firewall_req *req)
@@ -744,12 +764,12 @@ int ctl_firewall(int op, struct firewall_req *req)
 		unit->up, unit->force, unit->impulse_mode,
 		impulse_init_time, impulse_time,
 		impulse_fuzz);
-	    if (monitors) mon_write(buf,strlen(buf));
+	    if (monitors) mon_write(MONITOR_STATUS,buf,strlen(buf));
 
-	    pcountdown((unit->impulse_mode)?(unit->impulse.expected-tstamp):0);
-	    pcountdown(unit->force_etime-atime);
-	    pcountdown(next_alarm());
-	    if (monitors) mon_write("QUEUE\n",6);
+	    pcountdown(MONITOR_STATUS,(unit->impulse_mode)?(unit->impulse.expected-tstamp):0);
+	    pcountdown(MONITOR_STATUS,unit->force_etime-atime);
+	    pcountdown(MONITOR_STATUS,next_alarm());
+	    if (monitors) mon_write(MONITOR_QUEUE,"QUEUE\n",6);
 	    for (c=unit->connections->next; c!=unit->connections; c=c->next) {
 		if (c->timer.next == 0) continue;
                 addr.s_addr = c->id.id[1] + (c->id.id[2]<<8)
@@ -767,10 +787,10 @@ int ctl_firewall(int op, struct firewall_req *req)
                         "%-4s  %15s/%-5d  %15s/%-5d  ",
                         proto, saddr, sport,
                         daddr, dport);
-                if (monitors) mon_write(buf,strlen(buf));
-		pcountdown(c->timer.expected-tstamp);
+                if (monitors) mon_write(MONITOR_QUEUE,buf,strlen(buf));
+		pcountdown(MONITOR_QUEUE,c->timer.expected-tstamp);
 	    }
-	    if (monitors) mon_write("END QUEUE\n",10);
+	    if (monitors) mon_write(MONITOR_QUEUE,"END QUEUE\n",10);
 	    return 0;
 	}
 	return 0;
@@ -795,6 +815,7 @@ int ctl_firewall(int op, struct firewall_req *req)
 		    units[i].used = 1;
 		    units[i].force_etime = 0;
 		    units[i].impulse_etime = 0;
+		    units[i].waiting = 1;
 		    return i;
 		}
 	    return -1;	/* ERRNO */
@@ -821,6 +842,11 @@ int ctl_firewall(int op, struct firewall_req *req)
 	/* turn off the impulse generator */
 	del_timer(&unit->impulse);
 	return 0;
+    case IP_FW_WAIT:
+	return unit->waiting;
+    case IP_FW_RESET_WAITING:
+	unit->waiting = 1;
+	return 0;
     }
     return -1; /* ERRNO */
 }
