@@ -20,9 +20,10 @@ struct {
 	void (*reroute)();
 	void (*kill)();
 	void (*zombie)();
-} pcontrol[2] = {
+} pcontrol[3] = {
     {slip_start,slip_set_addrs,slip_dead,slip_stop,slip_reroute,slip_kill,slip_zombie},
     {ppp_start,ppp_set_addrs,ppp_dead,ppp_stop,ppp_reroute,ppp_kill,ppp_zombie},
+    {dev_start,dev_set_addrs,dev_dead,dev_stop,dev_reroute,dev_kill,dev_zombie},
 };
 
 
@@ -87,7 +88,8 @@ void trans_CONNECT(void)
 	if (dial_status == 0) {
 	    dial_failures = 0;
 	    redial_rtimeout = redial_timeout;
-	    finish_dial(); /* go into a CLOCAL connection */
+	    if (mode != MODE_DEV)
+	        finish_dial(); /* go into a CLOCAL connection */
 	    GOTO(STATE_START_LINK);
 	} else {
 	    syslog(LOG_INFO,"Connect script failed.");
@@ -104,7 +106,7 @@ void trans_CONNECT(void)
 		dial_failures = 0;
 		redial_rtimeout = redial_timeout;
 		current_retry_count = 0;
-		syslog(LOG_INFO,"To many dialing falures in a row. Blocking connetion.");
+		syslog(LOG_INFO,"Too many dialing failures in a row. Blocking connection.");
 	    }
 	    GOTO(STATE_CLOSE);
 	}
@@ -172,6 +174,7 @@ void trans_START_LINK(void)
 	/* Ok, we're up and running. */
 	GOTO(STATE_UP);
     }
+
     /* If we fall through then we haven't finished the setup yet. */
     if (state_timeout == 0) {
 	/* If we time out, then we're going to stop. We need to reroute
@@ -206,9 +209,8 @@ void act_UP(void)
     flush_timeout_queue();
     interface_up();
     ppp_half_dead = 0;
-#ifdef BUFFER_PACKETS
-    forward_buffer();
-#endif
+    if (buffer_packets)
+   	 forward_buffer();
     /* Once we get here, we might as well kill off any outstanding
      * FIFO requests */
     if (!use_req && req_pid) {
@@ -241,6 +243,8 @@ void trans_UP(void)
 take_link_down:
 	current_retry_count = 0;
 	flush_timeout_queue();
+	/* WARNING: You MUST do this before calling idle_filter_proxy(),
+           or the snoop socket will not bind properly! */
    	(*pcontrol[mode].reroute)();
 	idle_filter_proxy();
 	GOTO(STATE_STOP_LINK);
@@ -256,6 +260,8 @@ take_link_down:
     	if (two_way) flush_timeout_queue();
 	no_redial_delay = 1;
 	current_retry_count = died_retry_count;
+	/* WARNING: You MUST do this before calling idle_filter_proxy(),
+           or the snoop socket will not bind properly! */
    	(*pcontrol[mode].reroute)();
 	idle_filter_proxy();
 	GOTO(STATE_DISCONNECT);
@@ -292,11 +298,13 @@ void trans_HALF_DEAD(void)
 
 void act_DISCONNECT(void)
 {
-   dial_status = 0;
-   if (disconnector) {
-       reopen_modem();
-       fork_dialer(disconnector, modem_fd);
-   }
+    dial_status = 0;
+    if (disconnector) {
+        if (mode != MODE_DEV) {
+	    reopen_modem();
+            fork_dialer(disconnector, modem_fd);
+	}
+    }
 }
 void trans_DISCONNECT(void)
 {
@@ -318,7 +326,7 @@ void act_CLOSE(void)
 	duration += call_start_time;
         fprintf(acctfp,"%s: Disconnected. Call duration %d seconds.\n",
 	    tm,duration);
-        fprintf(acctfp,"      IP transmited %d bytes and received %d bytes.\n",
+        fprintf(acctfp,"      IP transmitted %d bytes and received %d bytes.\n",
 	    txtotal,rxtotal);
 	fclose(acctfp);
     }
@@ -412,11 +420,10 @@ struct {
 void output_state()
 {
     if (monitors) {
-	mon_write("STATE\n",6);
-	mon_write(trans[state].name,strlen(trans[state].name));
-	mon_write("\n",1);
+	mon_write(MONITOR_STATE,"STATE\n",6);
+	mon_write(MONITOR_STATE,trans[state].name,strlen(trans[state].name));
+	mon_write(MONITOR_STATE,"\n",1);
     }
-    /* if (mon_fp) fflush(mon_fp); */
 }
 
 void GOTO(int new_state)
