@@ -252,6 +252,8 @@ int open_modem()
     modem_fd = -1;
     dial_status = 0;
 
+    if (mode == MODE_DEV) return 0;
+
     if (req_pid) {
 	/* The user has specified a device. Use it, no search or lock needed. */
 	if ((modem_fd = open(req_dev, O_RDWR | O_NDELAY)) < 0) {
@@ -272,14 +274,14 @@ int open_modem()
 	     * we run out of devices to try.
 	     */
 
-	    if (lock_dev && lock(devices[i]) < 0)
+	    if (lock_dev && lock(current_dev) < 0)
 		continue;
 
 	    /* OK. Locked one, try to open it */
-	    if ((modem_fd = open(devices[i], O_RDWR | O_NDELAY)) >= 0)
+	    if ((modem_fd = open(current_dev, O_RDWR | O_NDELAY)) >= 0)
 		break;
 	    else {
-	       syslog(LOG_ERR,"Error opening device %s: %m",devices[i]);
+	       syslog(LOG_ERR,"Error opening device %s: %m",current_dev);
 	    }
 	    current_dev = 0;
 
@@ -308,7 +310,7 @@ int open_modem()
 	    die(1);
 	}
 
-	if (tcsetpgrp(modem_fd, pgrpid) < 0) {
+        if (tcsetpgrp(modem_fd, pgrpid) < 0) {
 	    syslog(LOG_ERR, "failed to set process group: %m");
 	    die(1);
 	}
@@ -353,6 +355,7 @@ void reopen_modem()
 
     if(debug&DEBUG_VERBOSE)
 	syslog(LOG_INFO,"Reopening modem device");
+
     close(modem_fd);
     sleep(1);
     if ((modem_fd = open(current_dev, O_RDWR | O_NDELAY)) < 0) {
@@ -394,6 +397,12 @@ void finish_dial()
  */
 void close_modem()
 {
+    if (mode == MODE_DEV) {
+	req_pid = 0;
+	modem_fd = -1 ;
+	return ;
+    }
+
     if (debug&DEBUG_VERBOSE)
         syslog(LOG_INFO,"Closing modem line.");
 
