@@ -9,14 +9,19 @@
 #include "diald.h"
 
 #define MAXLINELEN 1024
-#define MAXARGS 10
+#define MAXARGS 512
 
 
 /* Configuration variables */
+char device[10]; /* The Device tu use for MODE_DEV */
+char device_node[9];
+int  device_iface ;
 char **devices = 0;
 int device_count = 0;
 int inspeed = DEFAULT_SPEED;
+int window = 0;
 int mtu = DEFAULT_MTU;
+int mru = DEFAULT_MTU;
 char *connector = 0;
 char *disconnector = 0;
 char *local_ip = 0;
@@ -32,6 +37,17 @@ char *ip_down = 0;
 char *acctlog = 0;
 char *pidlog = "diald.pid";
 char *fifoname = 0;
+char *lock_prefix = LOCK_PREFIX;
+int pidstring = 1;
+char *run_prefix = RUN_PREFIX;
+char *path_route = PATH_ROUTE;
+char *path_ifconfig = PATH_IFCONFIG;
+char *path_bootpc = PATH_BOOTPC;
+char *path_pppd = PATH_PPPD;
+int buffer_packets = BUFFER_PACKETS;
+int buffer_size = BUFFER_SIZE;
+int buffer_fifo_dispose = BUFFER_FIFO_DISPOSE;
+int buffer_timeout = BUFFER_TIMEOUT;
 FILE *acctfp = 0;
 int mode = MODE_SLIP;
 int debug = 0;
@@ -48,6 +64,7 @@ int two_way = 0;
 int give_way = 0;
 int do_reroute = UNSAFE_ROUTING;
 int proxyarp = 0;
+int route_wait = 0;
 
 int connect_timeout = 60;
 int disconnect_timeout = 60;
@@ -64,6 +81,12 @@ int redial_backoff_start = -1;
 int redial_backoff_limit = 600;
 int dial_fail_limit = 0;
 
+#ifdef SIOCSKEEPALIVE
+int keepalive = 0;
+#endif
+#ifdef SIOCSOUTFILL
+int outfill = 0;
+#endif
 
 struct {
     char *str;
@@ -74,15 +97,21 @@ struct {
 } commands[] = {
     {"-f","<file>",1,0,read_config_file},
     {"-file","<file>",1,0,read_config_file},
+    {"include","<file>",1,0,read_config_file},
 /* Mode options */
-    {"-m","[ppp|slip|cslip|slip6|cslip6|aslip]",1,0,set_mode},
-    {"mode","[ppp|slip|cslip|slip6|cslip6|aslip]",1,0,set_mode},
+    {"-m","[ppp|slip|cslip|slip6|cslip6|aslip|dev]",1,0,set_mode},
+    {"mode","[ppp|slip|cslip|slip6|cslip6|aslip|dev]",1,0,set_mode},
+/* Debugging options */
+    {"debug","<debugmask>",1,&debug,set_int},
+    {"-daemon","",0,&daemon,clear_flag},
 /* general options */
     {"accounting-log","<f>",1,&acctlog,set_str},
     {"pidfile","<f>",1,&pidlog,set_str},
     {"fifo","<f>",1,&fifoname,set_str},
 /* Networking addressing and control options */
+    {"window","<n>",1,&window,set_int},
     {"mtu","<m>",1,&mtu,set_int},
+    {"mru","<m>",1,&mru,set_int},
     {"local","<ip-address>",1,&local_ip,set_str},
     {"remote","<ip-address>",1,&remote_ip,set_str},
     {"netmask","<ip-address>",1,&netmask,set_str},
@@ -94,10 +123,14 @@ struct {
     {"proxyarp","",0,&proxyarp,set_flag},
     {"ip-up","<script>",1,&ip_up,set_str},
     {"ip-down","<script>",1,&ip_down,set_str},
-    {"two-way","",0,&two_way,set_flag},
-    {"give-way","",0,&give_way,set_flag},
     {"reroute","",0,&do_reroute,set_flag},
     {"-reroute","",0,&do_reroute,clear_flag},
+#ifdef SIOCSKEEPALIVE
+    {"keepalive","<0-255>",1,&keepalive,set_int},
+#endif
+#ifdef SIOCSOUTFILL
+    {"outfill","<0-255>",1,&outfill,set_int},
+#endif
 /* Modem options */
     {"device","<device>",1,0,add_device},
     {"connect","<script>",1,&connector,set_str},
@@ -107,10 +140,25 @@ struct {
     {"modem","",0,&modem,set_flag},
     {"crtscts","",0,&crtscts,set_flag},
     {"rotate-devices","",0,&rotate_devices,set_flag},
-/* Debugging options */
-    {"debug","<debugmask>",1,&debug,set_int},
-    {"-daemon","",0,&daemon,clear_flag},
-/* Timeout control options */
+/* Configuration options */
+    {"lock-prefix","<path>",1,&lock_prefix,set_str},
+    {"pidstring","",0,&pidstring,set_flag},
+    {"-pidstring","",0,&pidstring,clear_flag},
+    {"run-prefix","<path>",1,&run_prefix,set_str},
+    {"path-route","<path>",1,&path_route,set_str},
+    {"path-ifconfig","<path>",1,&path_ifconfig,set_str},
+    {"path-bootpc","<path>",1,&path_bootpc,set_str},
+    {"path-pppd","<path>",1,&path_pppd,set_str},
+    {"buffer-packets","",0,&buffer_packets,set_flag},
+    {"-buffer-packets","",0,&buffer_packets,clear_flag},
+    {"buffer_size","<n>",1,&buffer_size,buffer_init},
+    {"buffer-fifo-dispose","",0,&buffer_fifo_dispose,set_flag},
+    {"-buffer-fifo-dispose","",0,&buffer_fifo_dispose,clear_flag},
+    {"buffer-timeout","<n>",1,&buffer_timeout,set_int},
+/* Timeouts and connection policy control */
+    {"route-wait","",0,&route_wait,set_flag},
+    {"two-way","",0,&two_way,set_flag},
+    {"give-way","",0,&give_way,set_flag},
     {"connect-timeout","<timeout>",1,&connect_timeout,set_int},
     {"disconnect-timeout","<timeout>",1,&disconnect_timeout,set_int},
     {"redial-timeout","<timeout>",1,&redial_timeout,set_int},
@@ -125,6 +173,7 @@ struct {
     {"redial-backoff-start","<count>",1,&redial_backoff_start,set_int},
     {"redial-backoff-limit","<time>",1,&redial_backoff_limit,set_int},
     {"dial-fail-limit","<count>",1,&dial_fail_limit,set_int},
+/* Filter rules */
     {"prule","<name> <protocol> <spec>",3,0,&parse_prule},
     {"var","<name> <spec>",2,0,&parse_var},
     {"restrict","<start-time> <end-time> <weekday> <day> <month>",5,0,&parse_restrict},
@@ -147,7 +196,9 @@ void init_vars()
     devices = 0;
     device_count = 0;
     inspeed = DEFAULT_SPEED;
+    window = 0;
     mtu = DEFAULT_MTU;
+    mru = DEFAULT_MTU;
     connector = 0;
     disconnector = 0;
     local_ip = 0;
@@ -178,6 +229,7 @@ void init_vars()
     two_way = 0;
     do_reroute = UNSAFE_ROUTING;
     proxyarp = 0;
+    route_wait = 0;
     connect_timeout = 60;
     disconnect_timeout = 60;
     redial_timeout = DEFAULT_DIAL_DELAY;
@@ -192,6 +244,23 @@ void init_vars()
     redial_backoff_start = -1;
     redial_backoff_limit = 600;
     dial_fail_limit = 0;
+    lock_prefix = LOCK_PREFIX;
+    pidstring = 1;
+    run_prefix = RUN_PREFIX;
+    path_route = PATH_ROUTE;
+    path_ifconfig = PATH_IFCONFIG;
+    path_bootpc = PATH_BOOTPC;
+    path_pppd = PATH_PPPD;
+    buffer_packets = BUFFER_PACKETS;
+    buffer_size = BUFFER_SIZE;
+    buffer_fifo_dispose = BUFFER_FIFO_DISPOSE;
+    buffer_timeout = BUFFER_TIMEOUT;
+#ifdef SIOCSKEEPALIVE
+    keepalive = 0;
+#endif
+#ifdef SIOCSOUTFILL
+    outfill = 0;
+#endif
 }
 
 void set_int(int *var, char **argv)
@@ -208,6 +277,8 @@ void set_mode(char **var, char **argv)
 {
     if (strcmp(argv[0],"ppp") == 0)
 	mode = MODE_PPP;
+    else if (strcmp(argv[0],"dev") == 0)
+	mode = MODE_DEV;
     else if (strcmp(argv[0],"slip") == 0)
 	mode = MODE_SLIP, slip_encap = 0;
     else if (strcmp(argv[0],"cslip") == 0)
@@ -219,7 +290,7 @@ void set_mode(char **var, char **argv)
     else if (strcmp(argv[0],"aslip") == 0)
 	mode = MODE_SLIP, slip_encap = 8;
     else
-	syslog(LOG_ERR,"Unknown mode %s.\nValid modes are: ppp, slip, cslip, slip6, cslip6, or aslip.",argv[0]);
+	syslog(LOG_ERR,"Unknown mode %s.\nValid modes are: dev, ppp, slip, cslip, slip6, cslip6, or aslip.",argv[0]);
 }
 
 void set_dslip_mode(char **var, char **argv)
@@ -263,6 +334,23 @@ void usage(void)
     exit(1);
 }
 
+void copy_pppd_args(int argc, char *argv[])
+{
+    int i;
+#ifdef NO_MEM_LEAKS
+    if (pppd_argv) {
+	while (i = 0; i < argc; i++)
+	    free(pppd_argv[i]);
+	free(pppd_argv);
+    }
+#endif
+    pppd_argv = malloc(sizeof(char **)*argc);
+    for (i = 0; i < argc; i++) {
+	pppd_argv[i] = strdup(argv[i]);
+    }
+    pppd_argc = argc;
+}
+
 void add_device(void *var, char **argv)
 {
     device_count++;
@@ -272,20 +360,20 @@ void add_device(void *var, char **argv)
 	devices = (char **)realloc(devices,sizeof(char *)*device_count);
 
     devices[device_count-1] = strdup(argv[0]);
-
 }
 
+
 void parse_args(int argc, char *argv[])
 {
     int i;
     struct stat st;
 
-    /* Get any devices from the initial arguments */
+    /* Get any tty devices from the initial arguments */
     for (i = 0; argc > 0 && stat(*argv,&st) == 0 && S_ISCHR(st.st_mode); i++) {
 	add_device(0,argv);
 	argc--, argv++;
     }
-
+    
     /*
      * Ok, parse the options list now,
      * command line options override/augment defaults
@@ -313,8 +401,8 @@ void parse_args(int argc, char *argv[])
 	}
     }
     /* Any remaining options are food for pppd */
-    pppd_argv = argv;
-    pppd_argc = argc+1;
+    if (++argc > 0)
+    	copy_pppd_args(argc,argv);
 }
 
 /* Parse the given options file */
@@ -334,7 +422,7 @@ void parse_options_file(char *file)
     }
     while (fscanf(fp,"%1024[^\n]\n",line) != -1) {
 	argc = 0;
-	for (s = line, argc = 0; *s || argc < MAXARGS; s++) {
+	for (s = line, argc = 0; *s && argc < MAXARGS; s++) {
 	    if (*s == ' ' || *s == '\t')
 		*s = 0;
 	    else if (*s == '#' && argc == 0) /* the line is a comment */
@@ -416,6 +504,8 @@ void parse_options_file(char *file)
 	    } else {
 	        (commands[i].parser)(commands[i].var,&argv[1]);
 	    }
+	} else if (strcmp("pppd-options",argv[0]) == 0) {
+		copy_pppd_args(argc-1,&argv[1]);
 	} else {
 	    syslog(LOG_ERR,"Unknown option '%s'",*argv);
 	}
@@ -429,17 +519,28 @@ void check_setup()
     struct stat st;
     int i;
     int flag = 0;
+    int temp ;
+    char *p ;
 
     if (device_count == 0)
 	flag = 1,
 	syslog(LOG_ERR,
 	    "No device specified. You must have at least one device!");
 
-    for (i = 0; i < device_count; i++)
-	if (stat(devices[i],&st) < 0 || !S_ISCHR(st.st_mode))
-	    flag = 1,
-	    syslog(LOG_ERR,
-		"Specified device '%s' not a character device.",devices[0]);
+    /* If for MODE_DEV then remove the /dev/ */
+    strcpy(device , strrchr(devices[0],'/')+1 );
+	
+    temp = strcspn(device, "0123456789" ) ;
+    strncpy(device_node , device , temp ) ;
+    p = device + temp + 1;
+    device_iface = atoi( p );
+
+    if (mode != MODE_DEV)
+	for (i = 0; i < device_count; i++)
+	    if (stat(devices[i],&st) < 0 || !S_ISCHR(st.st_mode))
+		flag = 1,
+		syslog(LOG_ERR,
+		    "Specified device '%s' not a character device.",devices[0]);
 
     if (!connector)
 	flag = 1,
