@@ -20,12 +20,16 @@
 #define PPPIOCGUNIT_2_2_0 _IOR('t', 86, int)
 #endif
 
+/* internal flag to shortcut repeated calls to setaddr */
+static int rx_count = -1;
+
 void ppp_start()
 {
     int pgrpid;
     char buf[24];
 
     link_iface = -1;
+    rx_count = -1;
     /* Run pppd directly here and set up to wait for the iface */
     link_pid = fork();
 
@@ -45,12 +49,16 @@ void ppp_start()
         argv[i++] = "mtu";
 	sprintf(buf,"%d",mtu);
 	argv[i++] = buf;
+        argv[i++] = "mru";
+	sprintf(buf,"%d",mru);
+	argv[i++] = buf;
 	if (netmask) {
 	    argv[i++] = "netmask";
 	    argv[i++] = netmask;
 	}
-	for (j = 0; j < pppd_argc; j++)
+	for (j = 0; j < pppd_argc; j++) {
 	    argv[i++] = pppd_argv[j];
+	}
 	argv[i++] = 0;
 
 	/* make sure pppd is the session leader and has the controlling
@@ -118,6 +126,14 @@ int ppp_set_addrs()
 	if (!(ifr.ifr_flags & IFF_UP))
 	    return 0;	/* interface is not up yet */
 
+	if (route_wait) {
+	    /* set the initial rx counter once the link is up */
+	    if (rx_count == -1) rx_count = ppp_rx_count();
+
+	    /* check if we got the routing packet yet */
+	    if (ppp_rx_count() == rx_count) return 0;
+	}
+
 	/* Ok, the interface is up, grab the addresses. */
 	if (ioctl(snoopfd, SIOCGIFADDR, (caddr_t) &ifr) == -1)
 		syslog(LOG_ERR,"failed to get ppp local address: %m");
@@ -141,7 +157,7 @@ int ppp_set_addrs()
 		syslog(LOG_ERR,"Attempting to auto adjust mtu.");
 		syslog(LOG_ERR,"Restart diald with mtu set to %d to avoid errors.",ifr.ifr_mtu);
 		mtu = ifr.ifr_mtu;
-	        proxy_config(local_ip,remote_ip);
+	        proxy_config(orig_local_ip,orig_remote_ip);
 	    }
 	}
 
@@ -155,19 +171,25 @@ int ppp_set_addrs()
 	    local_addr = laddr;
 	    syslog(LOG_INFO,"New addresses: local %s, remote %s.",
 		local_ip,remote_ip);
-	    proxy_config(local_ip,remote_ip);
 	    /* have to reset the proxy if we won't be rerouting... */
-	    if (!do_reroute)
-	        add_routes("sl",proxy_iface,local_ip,remote_ip,2000);
+	    if (!do_reroute) {
+		/* If we are rerouting, then we have a window without
+		 * routes here. The proxy_config calls ifconfig, which
+		 * clobbers all the routes.
+		 */
+	    	proxy_config(local_ip,remote_ip);
+    		set_ptp("sl",proxy_iface,remote_ip,1);
+	        add_routes("sl",proxy_iface,local_ip,remote_ip,1);
+	    }
 	}
 
 	/* This is redundant in normal operation, but if we
 	 * have to restart the link, then this is necessary...
 	 */
-	set_ptp("ppp",link_iface,remote_ip,500);
+	set_ptp("ppp",link_iface,remote_ip,0);
 
 	if (do_reroute)
-	     add_routes("ppp",link_iface,local_ip,remote_ip,500);
+	     add_routes("ppp",link_iface,local_ip,remote_ip,0);
 	return 1;
     }
     return 0;
@@ -175,15 +197,12 @@ int ppp_set_addrs()
 
 int ppp_dead()
 {
-    if (link_pid == 0) {
-	/* pppd is way too enthusiastic about deleting routes it had
-	 * nothing to do with creating. Therefore, we have to reestablish
-	 * the proxy routes here, including the point to point route!
-	 */
-	set_ptp("sl",proxy_iface,orig_remote_ip,2000);
-	add_routes("sl",proxy_iface,orig_local_ip,orig_remote_ip,2000);
-    	local_addr = inet_addr(orig_local_ip);
-    }
+    /* pppd is way too enthusiastic about deleting routes it had
+     * nothing to do with creating. Therefore, we have to reestablish
+     * the proxy routes here, including the point to point route!
+     */
+    if (link_pid == 0)
+	ppp_reroute();
     return (link_pid == 0);
 }
 
@@ -200,7 +219,7 @@ int ppp_route_exists()
     }
 
     while (fgets(buf,128,fp)) {
-	if (sscanf(buf,"%*s %*s %*s %*s %*s %*s %*s ppp%d",&device) == 8) {
+	if (sscanf(buf,"%*s %*s %*s %*s %*s %*s %*s ppp%d",&device) == 1) {
 	    if (device == link_iface) found = 1;
 	}
     }
@@ -208,6 +227,26 @@ int ppp_route_exists()
     return found;
 }
 
+int ppp_rx_count()
+{
+    char buf[128];
+    int packets = 0;
+    FILE *fp;
+    sprintf(buf,"%s ppp%d",PATH_IFCONFIG,link_iface);
+    if ((fp = popen(buf,"r"))==NULL) {
+        syslog(LOG_ERR,"Could not run command '%s': %m",buf);
+	return 0;	/* assume half dead in this case... */
+    }
+
+    while (fgets(buf,128,fp)) {
+	if (sscanf(buf," RX packets:%d",&packets) == 1) {
+	    break;
+	}
+    }
+    fclose(fp);
+    return packets;
+}
+
 void ppp_stop()
 {
     if (link_pid)
@@ -218,13 +257,19 @@ void ppp_stop()
 void ppp_reroute()
 {
     /* Restore the original proxy routing */
+    /* If there was a change in the proxy addresses
+     * (i.e., we are running with -reroute), then
+     * this will change introduce a window in which
+     * we loose our routes. There does not seem to be a way
+     * to avoid this. Sigh.
+     */
     proxy_config(orig_local_ip,orig_remote_ip);
-    set_ptp("sl",proxy_iface,orig_remote_ip,2000);
-    add_routes("sl",proxy_iface,orig_local_ip,orig_remote_ip,2000);
+    set_ptp("sl",proxy_iface,orig_remote_ip,1);
+    add_routes("sl",proxy_iface,orig_local_ip,orig_remote_ip,1);
     local_addr = inet_addr(orig_local_ip);
     /* If we did routing on the ppp link, remove it */
     if (do_reroute && link_iface != -1)
-    	del_routes("ppp",link_iface,local_ip,remote_ip,500);
+    	del_routes("ppp",link_iface,local_ip,remote_ip,0);
     link_iface = -1;
 }
 
