@@ -15,6 +15,8 @@ static int start_disc;
 static FILE *bootpfp;
 static PIPE bootp_pipe;
 
+static int rx_count = -1;
+
 
 static void start_bootp()
 {
@@ -99,7 +101,7 @@ static void slip_start_fail(unsigned long data)
    fail = 1;
 }
 
-struct timer_list failt;
+struct timer_lst failt;
 
 void slip_start(void)
 {
@@ -107,6 +109,8 @@ void slip_start(void)
     int disc, sencap;
     int res;
 
+    rx_count = -1;
+
     if (dynamic_addrs && !force_dynamic) {
 	if (debug&DEBUG_VERBOSE)
 	    syslog(LOG_INFO,"Fetching IP addresses from SLIP server");
@@ -148,6 +152,16 @@ void slip_start(void)
     if (ioctl(modem_fd, SIOCSIFENCAP, &slip_encap) < 0)
 	syslog(LOG_ERR,"Can't set encapsulation: %m"), die(1);
 
+#ifdef SIOCSKEEPALIVE
+    if (keepalive && (ioctl(modem_fd, SIOCSKEEPALIVE, &keepalive) < 0))
+      syslog(LOG_ERR, "Can't set keepalive: %m (ignoring error)");
+#endif
+
+#ifdef SIOCSOUTFILL
+    if (outfill && (ioctl(modem_fd, SIOCSOUTFILL, &outfill) < 0))
+      syslog(LOG_ERR, "Can't set outfill: %m (ignoring error)");
+#endif
+
     /* verify that it worked */
     if (ioctl(modem_fd, TIOCGETD, &disc) < 0)
 	syslog(LOG_ERR,"Can't get line discipline: %m"), die(1);
@@ -173,7 +187,7 @@ void slip_start(void)
     report_system_result(res,buf);
 
     /* Set the routing for the new slip interface */
-    set_ptp("sl",link_iface,remote_ip,500);
+    set_ptp("sl",link_iface,remote_ip,0);
 
     /* run bootp if it is asked for */
     if (dynamic_addrs && dynamic_mode == DMODE_BOOTP && !force_dynamic) start_bootp();
@@ -224,6 +238,13 @@ int slip_set_addrs()
     	waiting_for_bootp = 0;
     }
 
+    if (route_wait) {
+	/* set the initial rx counter once the link is up */
+	if (rx_count == -1) rx_count = slip_rx_count();
+
+	/* check if we got the routing packet yet */
+	if (slip_rx_count() == rx_count) return 0;
+    }
 
     /* redo the interface marking and the routing since BOOTP will change it */
     if (netmask) {
@@ -237,24 +258,28 @@ int slip_set_addrs()
     report_system_result(res,buf);
 
     /* Set the routing for the new slip interface */
-    set_ptp("sl",link_iface,remote_ip,500);
+    set_ptp("sl",link_iface,remote_ip,0);
 
     if (dynamic_addrs || force_dynamic) {
 	local_addr = inet_addr(local_ip);
-	proxy_config(local_ip,remote_ip);
 	/* have to reset the proxy if we won't be rerouting... */
-	if (!do_reroute)
-	    add_routes("sl",proxy_iface,local_ip,remote_ip,2000);
+	if (!do_reroute) {
+	    proxy_config(local_ip,remote_ip);
+    	    set_ptp("sl",proxy_iface,remote_ip,1);
+	    add_routes("sl",proxy_iface,local_ip,remote_ip,1);
+	}
     }
 
     if (do_reroute)
-        add_routes("sl",link_iface,local_ip,remote_ip,500);
+        add_routes("sl",link_iface,local_ip,remote_ip,0);
 
     return 1;
 }
 
 int slip_dead()
 {
+    if (dead)
+	slip_reroute();
     return (dead);
 }
 
@@ -272,15 +297,36 @@ void slip_reroute()
 {
     /* Restore the original proxy routing */
     proxy_config(orig_local_ip,orig_remote_ip);
-    set_ptp("sl",proxy_iface,orig_remote_ip,2000);
-    add_routes("sl",proxy_iface,orig_local_ip,orig_remote_ip,2000);
+    set_ptp("sl",proxy_iface,orig_remote_ip,1);
+    add_routes("sl",proxy_iface,orig_local_ip,orig_remote_ip,1);
     local_addr = inet_addr(orig_local_ip);
     /* If we did routing on the slip link, remove it */
     if (do_reroute && link_iface != -1) /* just in case we get called twice */
-    	del_routes("sl",link_iface,local_ip,remote_ip,500);
+    	del_routes("sl",link_iface,local_ip,remote_ip,0);
     link_iface = -1;
 }
 
+int slip_rx_count()
+{
+    char buf[128];
+    int packets = 0;
+    FILE *fp;
+    sprintf(buf,"%s sl%d",PATH_IFCONFIG,link_iface);
+    if ((fp = popen(buf,"r"))==NULL) {
+        syslog(LOG_ERR,"Could not run command '%s': %m",buf);
+	return 0;	/* assume half dead in this case... */
+    }
+
+    while (fgets(buf,128,fp)) {
+	if (sscanf(buf," RX packets:%d",&packets) == 1) {
+	    break;
+	}
+    }
+    fclose(fp);
+    return packets;
+}
+
+
 /* Dummy proc. This should never get called */
 void slip_kill()
 {
