@@ -14,9 +14,11 @@
 #include <sys/times.h>
 #include <unistd.h>
 
-static struct timer_list head = {&head,&head,0,0,0,0};
+static struct timer_lst head = {&head,&head,0,0,0,0};
 static int in_alarm = 0;
 
+static int block_level = 0;
+
 /*
  * The following gets a time stamp based on the number of system
  * clock ticks since the system has been up.
@@ -31,7 +33,7 @@ unsigned long timestamp()
    return times(&buf)/CLK_TCK;
 }
 
-void init_timer(struct timer_list * timer)
+void init_timer(struct timer_lst * timer)
 {
     timer->next = NULL;
     timer->prev = NULL;
@@ -45,6 +47,11 @@ void init_timer(struct timer_list * timer)
 void block_timer()
 {
     sigset_t mask;
+    if (block_level) {
+	block_level++;
+	return;
+    }
+    block_level++;
     if (!in_alarm) {
         sigemptyset(&mask);
         sigaddset(&mask,SIGALRM);
@@ -55,6 +62,8 @@ void block_timer()
 void unblock_timer()
 {
     sigset_t mask;
+    block_level--;
+    if (block_level) return;
     if (!in_alarm) {
         sigemptyset(&mask);
         sigaddset(&mask,SIGALRM);
@@ -69,9 +78,9 @@ void unblock_timer()
  * matched the original kernel type definition I was using.
  */
 
-void add_timer(struct timer_list *timer)
+void add_timer(struct timer_lst *timer)
 {
-    struct timer_list *c;
+    struct timer_lst *c;
     struct itimerval itime;
     unsigned long atime;
     block_timer();
@@ -103,7 +112,7 @@ void add_timer(struct timer_list *timer)
     unblock_timer();
 }
 
-int del_timer(struct timer_list *timer)
+int del_timer(struct timer_lst *timer)
 {
     unsigned long atime;
     block_timer();
@@ -139,8 +148,8 @@ int del_timer(struct timer_list *timer)
 void alrm_timer(int sig)
 {
     struct itimerval itime;
-    struct timer_list *cn;
-    struct timer_list *c = head.next;
+    struct timer_lst *cn;
+    struct timer_lst *c = head.next;
     unsigned long atime = timestamp();
     unsigned long now;
 
@@ -198,7 +207,7 @@ void sleep_wakeup(int *var)
 unsigned int sleep(unsigned int secs)
 {
     static int asleep;
-    struct timer_list sleept;
+    struct timer_lst sleept;
     init_timer(&sleept);
     sleept.data = (int)&asleep;
     sleept.function = (void *)(unsigned long)sleep_wakeup;
